/* Vertex shader */

#define MAIN_DRAWING 0
#define TARGET_RESPONSE_BLEND 1
#define TARGET_RESPONSE_CIRCLE 11
#define ENVELOPE_FRONT 6
#define ENVELOPE_BACK 7
#define HEAT_MAP 10


uniform float width;
uniform float height;
uniform int stage;
uniform float level;

uniform vec4 RGB;								// -> MAIN DRAWING RGBA
uniform int MODE;								// -> DRAWING MODE - SEE #defines
uniform vec4 METRICS;							// -> SIZES FOR DRAWING TARGET OUTLINES
uniform vec4 BOUNDING;							// -> BOUNDING BOX L,T,R,B FOR CLIPPING - CLI/EDITOR
uniform int USE_BOUNDING;						// -> ENABLE SWITCH

uniform vec4 IMG_POST_CONTRAST;					// -> BRIGHTNESS & CONTRAST
uniform vec4 IMG_POST_MONOCHROME_BLEND;			// -> MONOCHROME BLEND PORTIONS
uniform vec4 IMG_POST_RGB_TINT;					// -> COLOUR TINT
uniform vec4 IMG_POST_HSL;						// -> HSL

uniform int CIRCLE_MODE;

float RGB2HUE(in float RR, in float GG, in float BB) {
	float MMAX = max(max(RR, GG), BB);
	float MMIN = min(min(RR, GG), BB);
	float l = (MMAX + MMIN) / 2.0;
	float h = l;
	float s = l;
	float d = MMAX - MMIN;

	if (MMAX == MMIN) {
		h = s = 0.0; // achromatic
	}
	else {


		if (l > 0.5) {
			s = d / (2.0 - MMAX - MMIN);
		}
		else {
			s = d / (MMIN + MMIN);
		}

		//s = l > 0.5 ? d / (2.0 - MMAX - MMIN) : d / (MMIN + MMIN);

		if (MMAX == RR) {
			float f;
			if (GG < BB) {
				f = 6.0;
			}
			else {
				f = 0.0;
			}
			h = (GG - BB) / d + f;
		}

		if (MMAX == GG) {
			h = (BB - RR) / d + 2.0;
		}


		if (MMAX == BB) {
			h = (RR - GG) / d + 4.0;
		}
		h = h / 6.0;
	}

	return h;
};

void RGB2HSL(in float RR, in float GG, in float BB, out float HH, out float SS, out float LL) {
	float MMAX = max(max(RR, GG), BB);
	float MMIN = min(min(RR, GG), BB);
	LL = (MMAX + MMIN) / 2.0;
	HH = LL;
	SS = LL;


	if (MMAX == MMIN) {
		HH = SS = 0.0; // achromatic
	}
	else {

		float d = MMAX - MMIN;

		// s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
		if (LL > 0.5) {
			SS = d / (2.0 - MMAX - MMIN);
		}
		else {
			SS = d / (MMAX + MMIN);
		}

		//  case r: h = (g - b) / d + (g < b ? 6 : 0); break;
		if (MMAX == RR) {

			float f;
			if (GG < BB) {
				f = 6.0;
			}
			else {
				f = 0.0;
			}
			HH = (GG - BB) / d + f;
		}

		// case g: h = (b - r) / d + 2; break;
		if (MMAX == GG) {
			HH = (BB - RR) / d + 2.0;
		}

		// case b: h = (r - g) / d + 4; break;
		if (MMAX == BB) {
			HH = (RR - GG) / d + 4.0;
		}
		HH = HH / 6.0;
	}


};

float HUE2RGB(in float p, in float q, in float t) {

	if (t < 0.0) t += 1.0;
	if (t > 1.0) t -= 1.0;
	if (t < 1.0 / 6.0) return p + (q - p) * 6.0 * t;
	if (t < 1.0 / 2.0) return q;
	if (t < 2.0 / 3.0) return p + (q - p) * (2.0 / 3.0 - t) * 6.0;
	return p;
}

void HLS2RGB(in float hue, in float saturation, in float lightness, out float red, out float green, out float blue) {

	float q;
	float p;

	if (saturation == 0.0) {	// achromatic
		red = lightness;
		green = lightness;
		blue = lightness;
	}
	else {
		q = lightness < 0.5 ? lightness * (1.0 + saturation) : lightness + saturation - lightness * saturation;
		p = 2.0 * lightness - q;
		red = HUE2RGB(p, q, hue + 1.0 / 3.0);
		green = HUE2RGB(p, q, hue);
		blue = HUE2RGB(p, q, hue - 1.0 / 3.0);
	}
}

void main(void)
{
	vec4 v = vec4(gl_Vertex);
	vec4 c, b;

	float h = (2.0 / 3.0) - (1.0 - (v.y / height)) * (2.0 / 3.0);
	float red;
	float green;
	float blue;
	float H;
	float S;
	float L;

	c.r = red;
	c.g = green;
	c.b = blue;
	c.a = 1.0;

	b.a = 0.7;
	b.r = 0.0;
	b.g = 0.0;
	b.b = 0.0;

	if (stage == 0) {
	}
	else {

		c.r = 1.0;
		c.g = 1.0;
		c.b = 1.0;
		c.a = 1.0;

	}
	float A;

	float x1;
	float x2;
	float d;
	int THE_MODE = MODE;
	if (MODE == TARGET_RESPONSE_BLEND) {
		THE_MODE = MODE + (CIRCLE_MODE * 10);

	}

	switch (THE_MODE) {
	case TARGET_RESPONSE_BLEND:
		// BLENDED CIRCLE
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		A = RGB[3] * 1.0;
		c[3] = (2.0 * A) - (2.0 * A * sqrt((v[0] * v[0]) + (v[1] * v[1])) / 100.0);

		// SCALE THE VERTEX
		v[0] *= METRICS[2] * 0.01 * METRICS[3] * 1.0;
		v[1] *= METRICS[2] * 0.01 * METRICS[3] * 1.0;

		// POSITION THE VERTEX
		v[0] += METRICS[0];
		v[1] += METRICS[1];

		// GIVE IT A COLOUR
		gl_FrontColor = c;
		gl_BackColor = c;

		break;
	case TARGET_RESPONSE_CIRCLE:
		// FLAT CIRCLE
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		A = RGB[3] * 1.0;
		c[3] = 0.2;

		// SCALE THE VERTEX
		v[0] *= METRICS[2] * 0.01 * METRICS[3] * 1.0;
		v[1] *= METRICS[2] * 0.01 * METRICS[3] * 1.0;

		// POSITION THE VERTEX
		v[0] += METRICS[0];
		v[1] += METRICS[1];

		// GIVE IT A COLOUR
		gl_FrontColor = c;
		gl_BackColor = c;

		break;
	case 2:
	{
		// OLD DRAW THE OUTLINE FOR RESPONSE ...

		A = RGB[3] * 1.0;
		x1 = v[0];// - METRICS[3];
		x2 = v[1];// - METRICS[3];

		d = sqrt((x1 * x1) + (x2 * x2));
		d = abs(100 - d);
		//		d = d*1;

		c[3] = 1.0 - d;

		v[0] *= METRICS[3] * 0.01;
		v[1] *= METRICS[3] * 0.01;
		v[0] += METRICS[0];
		v[1] += METRICS[1];

		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		c[3] = RGB[3];

		gl_FrontColor = c;
		gl_BackColor = c;
	}
	break;
	case 5:
		// draw a picture

		break;
	case ENVELOPE_FRONT:

		// DRAW MOVING ENVELOPE RESPONSE
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		c[3] = RGB[3];
		c.a = 0.3 + (METRICS[3] * 0.7);
		if (METRICS[3] > 0.9999) {
			c[0] = 1.0;
			c[1] = 0.0;
			c[2] = 0.0;
			c[3] = 1.0;
		}
		gl_FrontColor = c;
		gl_BackColor = c;
		break;
	case ENVELOPE_BACK:
		// BACK OF ENVELOPE -> FLASHES RED IF OVERLOAD
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		c[3] = RGB[3];
		//c.a = 0.3 + (METRICS[3] * 0.7);
		if (METRICS[3] > 0.9999) {
			c[0] = 1.0;
			c[1] = 0.0;
			c[2] = 0.0;
			c[3] = 1.0;
		}
		gl_FrontColor = c;
		gl_BackColor = c;
		break;
	case MAIN_DRAWING:
		c = RGB;


		gl_FrontColor = c;
		gl_BackColor = c;

		break;

	case 99:
		// INVERTED

		c[0] = 1.0 - RGB[0];
		c[1] = 1.0 - RGB[1];
		c[2] = 1.0 - RGB[2];
		c[3] = RGB[3];

		// TEMP B&W
		gl_FrontColor = c;
		gl_BackColor = c;
		break;

	case HEAT_MAP:
		float D = RGB[3];
		float ALPHA = 0.5;
		if (D < 0.001) {
			ALPHA = 0.0;
		}
		D = sqrt(D);
		// MAGIC NUMBERS ...
		D *= 2.4;
		D += 0.67;
		D -= 1.0;
		D *= -1.0;
		D += 0.2;
		D = max(-1.0, D);
		D = min(1.0, D);

		// MAKE A HSL GRADIENT
		float LL = 1.0;
		LL = min(LL, 1.0);
		LL = max(LL, 0.0);
		D -= RGB[2];
		if (D < -1.0) {
			D += 1.0;
		}

		HLS2RGB(D, LL, 0.5, red, green, blue);

		c[0] = red;
		c[1] = green;
		c[2] = blue;
		c[3] = ALPHA;

		gl_FrontColor = c;
		gl_BackColor = c;

		break;
	case 12:
		// FEEDBACK CABLES
		// VALUE IN ALPHA CHANNEL DETERMINES HUE -> ALPHA is IN dB
		// ANTHING OVER -0dB IS RED, ELSE 0->0.5 GREEN, THROUGH ORANGE
		float HH = RGB[3];
		float AA = 1.0;

		//if(HH < -60.0){
		//}else{

		HH = min(HH, 0.0);
		HH = max(-60.0, HH);
		HH /= -120.0;

		HH = max(HH, 0.0);

		// ALPHA VALUE FALL-OFF BELOW -30 FROM 1 DOWN TO 0.5;

		if (HH < 0.5) {
			AA = 1.0 - (1.8 * HH);
		}
		if (HH > 0.499) {
			AA = 0.2;
		}

		//};
		HLS2RGB(HH, 1.0, 0.5, red, green, blue);
		//HLS2RGB(0.5,0.5,0.5,red,green,blue);
		c[0] = red;
		c[1] = green;
		c[2] = blue;
		c[3] = AA;

		gl_FrontColor = c;
		gl_BackColor = c;

		break;

	}

	// -> CROP (IF ANY)
	if (USE_BOUNDING == 1) {
		vec4 P = gl_ModelViewProjectionMatrix * v;
		P[0] *= (width * 0.5);
		P[1] *= -(height * 0.5);
		if (P[0]< BOUNDING[0] || P[0] > BOUNDING[2]) {
			//c[3] = 0.0;
			gl_FrontColor[3] = 0.0;
		}
		if (P[1] < BOUNDING[1] || P[1] > BOUNDING[3]) {
			//c[3] = 0.0;
			gl_FrontColor[3] = 0.0;
		}
	}


	// -> POST COLOUR
	c = gl_FrontColor;
	
	// -> SAVE ALPHA BECAUSE OF MONOCHROME PRE-MULTIPLICATION OF VALUES
	A = c.a;

	// -> MONOCHROME
	if (IMG_POST_MONOCHROME_BLEND[3] > 0.0) {
		float TOTAL_BLEND = IMG_POST_MONOCHROME_BLEND.r + IMG_POST_MONOCHROME_BLEND.g + IMG_POST_MONOCHROME_BLEND.b;
		c.r = (RGB.r * IMG_POST_MONOCHROME_BLEND.r * RGB.a) + (RGB.g * IMG_POST_MONOCHROME_BLEND.g * RGB.a) + (RGB.b * IMG_POST_MONOCHROME_BLEND.b * RGB.a);
		c.r /= TOTAL_BLEND;
		c.g = c.r;
		c.b = c.g;
	}

	// -> BRIGHTNESS AND CONTRAST
	if (IMG_POST_CONTRAST[3] > 0.0) {
		c -= IMG_POST_CONTRAST[2];
		c *= IMG_POST_CONTRAST[1];
		c += IMG_POST_CONTRAST[2];
		c += IMG_POST_CONTRAST[0];
	}

	// -> COLOUR TINT
	if (IMG_POST_RGB_TINT[3] > 0.0) {
		c.r *= IMG_POST_RGB_TINT[0];
		c.g *= IMG_POST_RGB_TINT[1];
		c.b *= IMG_POST_RGB_TINT[2];
	}

	// -> HSL
	if (IMG_POST_HSL[3] > 0.0) {
		float _H;
		float _S;
		float _L;
		RGB2HSL(c.r, c.g, c.b, _H, _S, _L);
		_H += IMG_POST_HSL[0];
		_S *= IMG_POST_HSL[1];
		_L *= IMG_POST_HSL[2];
		HLS2RGB(_H, _S, _L, red, green, blue);
		red = min(1.0, red);
		green = min(1.0, green);
		blue = min(1.0, blue);
		c.r = red;
		c.g = green;
		c.b = blue;
	}

	c.a = A;

	gl_FrontColor = c;
	gl_Position = gl_ModelViewProjectionMatrix * v;

}

